@@ -1,6 +1,6 @@
 /* gnomevfs-ls.c - Test for open_dir(), read_dir() and close_dir() for gnome-vfs
 
-   Copyright (C) 2003, Red Hat
+   Copyright (C) 2003, 2005, Red Hat
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -20,13 +20,31 @@
    Author: Bastien Nocera <hadess@hadess.net>
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <glib.h>
 #include <locale.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-static char *directory;
+#include "authentication.c"
 
-static void show_data (gpointer item, gpointer no_item);
-static void list (void);
+static gboolean timing = FALSE;
+static gboolean quiet = FALSE;
+static gboolean access_rights = FALSE;
+
+static GOptionEntry entries[] = 
+{
+	{ "time", 't', 0, G_OPTION_ARG_NONE, &timing, "Time the directory listening operation", NULL },
+	{ "quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet, "Do not output the stat information (useful in conjunction with the --time)", NULL},
+	{ "access-rights", 'a', 0, G_OPTION_ARG_NONE, &access_rights, "Get access rights", NULL},
+
+	{ NULL }
+};
+
+static void show_data (gpointer item, const char *directory);
+static void list (const char *directory);
 
 static const gchar *
 type_to_string (GnomeVFSFileType type)
@@ -54,7 +72,7 @@ type_to_string (GnomeVFSFileType type)
 }
 
 static void
-show_data (gpointer item, gpointer no_item)
+show_data (gpointer item, const char *directory)
 {
 	GnomeVFSFileInfo *info;
 	char *path;
@@ -63,7 +81,7 @@ show_data (gpointer item, gpointer no_item)
 
 	path = g_strconcat (directory, "/", info->name, NULL);
 
-	g_print ("%s\t%s%s%s\t(%s, %s)\tsize %ld\tmode %04o\n",
+	g_print ("%s\t%s%s%s\t(%s, %s)\tsize %" GNOME_VFS_SIZE_FORMAT_STR "",
 			info->name,
 			GNOME_VFS_FILE_INFO_SYMLINK (info) ? " [link: " : "",
 			GNOME_VFS_FILE_INFO_SYMLINK (info) ? info->symlink_name
@@ -71,22 +89,56 @@ show_data (gpointer item, gpointer no_item)
 			GNOME_VFS_FILE_INFO_SYMLINK (info) ? " ]" : "",
 			type_to_string (info->type),
 			info->mime_type,
-			(glong) info->size,
-			info->permissions);
+			info->size);
 
+	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS) {
+			g_print ("\tmode %04o", info->permissions);
+	}
+
+	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_ACCESS) {
+		g_print ("\t");
+		
+		if (info->permissions & GNOME_VFS_PERM_ACCESS_READABLE) {
+			g_print ("r");
+		}
+		
+		if (info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE) {
+			g_print ("w");
+		}
+		
+		if (info->permissions & GNOME_VFS_PERM_ACCESS_EXECUTABLE) {
+			g_print ("x");
+		}
+	}
+
+	g_print ("\n");
+	
 	g_free (path);
 }
 
 void
-list (void)
+list (const char *directory)
 {
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
 	GnomeVFSDirectoryHandle *handle;
+	GnomeVFSFileInfoOptions options;
+	GTimer *timer;
+	
+	timer = NULL;
+	if (timing) {
+		timer = g_timer_new ();
+		g_timer_start (timer);	
+	}
+	
+	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+			| GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 
-	result = gnome_vfs_directory_open (&handle, directory,
-			GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-			| GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (access_rights) {
+		options |= GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
+	}
+	
+	result = gnome_vfs_directory_open (&handle, directory, options);
 
 	if (result != GNOME_VFS_OK) {
 		g_print("Error opening: %s\n", gnome_vfs_result_to_string
@@ -96,9 +148,11 @@ list (void)
 
 	info = gnome_vfs_file_info_new ();
 	while ((result = gnome_vfs_directory_read_next (handle, info)) == GNOME_VFS_OK) {
-		show_data ((gpointer) info, NULL);
+		if (!quiet) {
+			show_data ((gpointer) info, directory);
+		}
 	}
-
+	
 	gnome_vfs_file_info_unref (info);
 
 	if ((result != GNOME_VFS_OK) && (result != GNOME_VFS_ERROR_EOF)) {
@@ -109,32 +163,60 @@ list (void)
 	result = gnome_vfs_directory_close (handle);
 
 	if ((result != GNOME_VFS_OK) && (result != GNOME_VFS_ERROR_EOF)) {
-		g_print ("Error: %s\n", gnome_vfs_result_to_string (result));
+		g_print ("Error closing: %s\n", gnome_vfs_result_to_string (result));
 		return;
 	}
 
+	if (timing) {
+		gdouble duration;
+		gulong  msecs;
+		
+		g_timer_stop (timer);
+		
+		duration = g_timer_elapsed (timer, &msecs);
+		g_timer_destroy (timer);
+		g_print ("Total time: %.16e\n", duration);
+	}
+
+	
 }
 
 int
 main (int argc, char *argv[])
 {
-  
+  	GError *error;
+	GOptionContext *context;
+
   	setlocale (LC_ALL, "");
 
 	gnome_vfs_init ();
 
+	command_line_authentication_init ();
+
+	error = NULL;
+	context = g_option_context_new ("- list files at <uri>");
+  	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+  	g_option_context_parse (context, &argc, &argv, &error);
+
 	if (argc > 1) {
-		directory = argv[1];
+		int i;
+
+		for (i = 1; i < argc; i++) {
+			char *directory;
+			directory = gnome_vfs_make_uri_from_shell_arg (argv[i]);
+			list (directory);
+			g_free (directory);
+		}
 	} else {
-		char *tmp;
+		char *tmp, *directory;
 
 		tmp = g_get_current_dir ();
-		directory = gnome_vfs_escape_path_string (tmp);
+		directory = gnome_vfs_get_uri_from_local_path (tmp);
+		list (directory);
 		g_free (tmp);
+		g_free (directory);
 	}
 
-	list ();
-
 	gnome_vfs_shutdown ();
 	return 0;
 }
